class NotLoggedIn (Exception):
    def __init__(self):
        pass
    def __str__(self):
        return "Call puckdns.API.login(<username>, <password>) to login to the puck dns server"

class LoginFailed(Exception):
    def __init__(self, username):
        self.__username = username
    def __str__(self):
        return f"(Re)login to puck dns service failed.\nPlease check ur user credentials giving to user: {self.__username}."

class PuckDnsError(Exception):
    def __init__(self, expectedMsg, recvMsg, url):
        self.__expectedMsg = expectedMsg
        self.__recvMsg = recvMsg
        self.__url = url
    def __str__(self):
        return f"Error during call of {self.__url}.\nPuck dns service throws following error:\n\'{self.__recvMsg}\'.\nExpected Message: \'{self.__expectedMsg}\'"

class PuckDnsWrongMsg(Exception):
    def __init__(self, expectedMsg, recvMsg, url):
        self.expectedMsg = expectedMsg
        self.recvMsg = recvMsg
        self.url = url
    def __str__(self):
        return f"Wrong behavior from puck dns service during call of '{self.url}\':\n Excepted Message: \'{self.expectedMsg}\'\nGot: \'{self.recvMsg}\'"



from xml.dom import minidom
from html.parser import HTMLParser

class PuckDnsParser(HTMLParser):
    """Help util to find 'DOM part' of response"""
    def handle_starttag(self, tag, attrs):
        if tag == "table":
            self.tableStart = self.getpos()[1]
        elif tag == "span":
            if attrs == [('id', 'error'), ('class', 'error')]:
                errorStart = self.getpos()[1] + len ('<span id="error" class="error">')
                self.errormsg = self.rawdata[errorStart:self.rawdata.find("</span>", errorStart)]
            elif attrs == [('id', 'message'), ('class', 'message')]:
                msgStart = self.getpos()[1] + len('<span id="message" class="message">')
                self.infomsg = self.rawdata[msgStart:self.rawdata.find("</span>", msgStart)]

    def handle_endtag(self, tag):
        if tag == "table":
            self.table = minidom.parseString(self.rawdata[self.tableStart:self.getpos()[1] + len("</table>")])



from socket import gethostbyname
from puckdns.puckdnsExceptions import *
import puckdns
import os
import unittest

TESTDOMAINS = ["puckdns-py-api.ml", "puckdns2-py-api.ml", "puckdns3-py-api.ml"]


class A_TestPuckDnsApiLogin(unittest.TestCase):
    def setUp(self):
        self.puck = puckdns.API()
    
    def tearDown(self):
        del self.puck

    def test_instance(self):
        self.assertTrue(isinstance(self.puck, puckdns.API))

    def test_a_tests_working(self):
        self.assertTrue("FOO".isupper())

    def test_env_username_set(self):
        self.assertTrue('PUCKDNS_USERNAME' in os.environ)

    def test_env_password_set(self):
        self.assertTrue('PUCKDNS_PASSWORD' in os.environ)

    def test_env_domain_set(self):
        self.assertTrue('PUCKDNS_DOMAIN' in os.environ)

    def test_not_logged_in_exception(self):
        self.assertRaises(NotLoggedIn, self.puck.getDomains)
        self.assertFalse(self.puck.isLoggedIn())

    def test_wrong_creds_exception(self):
        self.assertRaises(LoginFailed, self.puck.login,
                          os.environ['PUCKDNS_USERNAME'], "password")
        self.assertFalse(self.puck.isLoggedIn())

    def test_login_logout(self):
        self.assertIsNone(self.puck.login(
            os.environ['PUCKDNS_USERNAME'], os.environ['PUCKDNS_PASSWORD']))
        self.assertTrue(self.puck.isLoggedIn())
        self.puck.logout()
        self.test_not_logged_in_exception()


class B_PuckDnsApi(unittest.TestCase):
    HOST_IP = gethostbyname(os.environ['PUCKDNS_DOMAIN'])

    def setUp(self):
        self.puck = puckdns.API()
        self.puck.login(os.environ['PUCKDNS_USERNAME'],
                        os.environ['PUCKDNS_PASSWORD'])
        self.assertTrue(self.puck.isLoggedIn())

    def tearDown(self):
        self.puck.logout()
        self.assertFalse(self.puck.isLoggedIn())
        del self.puck

    def test_a_tests_working(self):
        self.assertTrue("FOO".isupper())

    def test_get_domains(self):
        self.assertEqual(self.puck.getDomains(), [])

    def test_add_del_domain(self):
        self.assertEqual(self.puck.getDomains(), [])
        self.puck.addDomain(self.HOST_IP, "puckdns-py-api.ml")
        self.assertEqual(self.puck.getDomains(), ["puckdns-py-api.ml"])
        self.puck.delDomain("puckdns-py-api.ml")
        self.assertEqual(self.puck.getDomains(), [])

    def test_add_del_domains(self):
        self.assertEqual(self.puck.getDomains(), [])
        self.puck.addDomains(self.HOST_IP, TESTDOMAINS)
        self.assertEqual(self.puck.getDomains(), TESTDOMAINS)
        self.puck.delDomains(TESTDOMAINS)
        self.assertEqual(self.puck.getDomains(), [])


if __name__ == '__main__':
    unittest.main(verbosity=2)



import setuptools
import os
import sys

# Get Version
sys.path.append(os.path.dirname(__file__))
import versioneer

__VERSION__ = versioneer.get_version()

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    version=__VERSION__,
    cmdclass=versioneer.get_cmdclass(),
    name="puckdns",
    author="Snake-Whisper",
    author_email="snake-whisper@web-utils.eu",
    description="Python API for the great free DNS Service \"PUCK\" from Daniel J. Luke (http://puck.nether.net/dns)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Snake-Whisper/puckdns",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    python_requires='>=3.6',
    command_options={
        'build_sphinx': {
            'version': ('setup.py', __VERSION__),
            'release': ('setup.py', __VERSION__),
            'source_dir': ('setup.py', 'docs')}},
)


from urllib import request
from urllib.parse import urlencode
from http.cookiejar import CookieJar
from .puckdnsExceptions import *
from .puckDnsParser import PuckDnsParser

class API():
    """API for working with https://puck.nether.net/dns"""

    __loggedIn = False
    __cookiejar = CookieJar()
    __HTTPRequestHandler = request.build_opener(request.HTTPCookieProcessor(__cookiejar))
    url = "https://puck.nether.net/dns/dnsinfo"

    def login(self, username, password):
        """Login to puck dns service with given username and password"""

        req = request.Request("https://puck.nether.net/dns/login", urlencode({'username': username, 'password': password, 'submit':'Login'}).encode())
        with self.__HTTPRequestHandler.open(req) as answer:
            if answer.url != 'https://puck.nether.net/dns/dnsinfo':
                raise LoginFailed(username)
            self.__loggedIn = True
            self.__username = username
            self.__pwd = password
    
    def isLoggedIn(self):
        return self.__loggedIn

    def logout(self):
        """Logout from puck dns service"""
        if self.__loggedIn:
            self._logout()

    def _logout(self):
        """Force logout from puck dns service"""
        req = request.Request("https://puck.nether.net/dns/logout")
        self.__HTTPRequestHandler.open(req)
        self.__loggedIn = False

    def __runTests (self):
        """Check preconditions like successful login before performing requests to puck dns service"""
        if not self.__loggedIn:
            raise NotLoggedIn()
    
    def __makeRequest(self, location, expectedMsg, payload=None):
        self.__runTests()
        
        url = self.url + location
        if payload:
            req = request.Request(url, urlencode(payload).encode())
        else:
            req = request.Request(url)
        answer = self.__HTTPRequestHandler.open(req)
        if answer.url == "https://puck.nether.net/dns/login":
            self.login(self.__username, self.__pwd)
            answer = self.__HTTPRequestHandler(req)
        parser = PuckDnsParser()
        parser.feed(answer.read().decode().replace("\n", ""))
        if parser.errormsg != '':
            raise PuckDnsError(expectedMsg, parser.errormsg, url)
        if parser.infomsg != expectedMsg:
            raise PuckDnsWrongMsg(expectedMsg, parser.infomsg, url)
        return parser

    def getDNSInfoTD (self):
        """Returns extracted table data from puck dns service"""
        parser = self.__makeRequest("", "")
        return parser.table.getElementsByTagName("td")

    def getDomains (self):
        """Returns registered domains from puck dns service"""
        tdList = self.getDNSInfoTD()
        return [tdList[i].firstChild.data for i in range(len(tdList)) if i%7 == 2] #from 7 fields in able the 3rd one -> 0,1,2

    def setIP (self, domain, ip):
        """Sets IP address for specific domains from puck dns service"""
        payload = {"domainname": domain, "masterip": ip, "aa": "Y", "submit": "Submit"}
        self.__makeRequest(f"/edit/{domain}", "Domain successfully edited.", payload)

    def getIP (self, domain):
        """Gets IP address for specific domains from puck dns service"""
        tdList = self.getDNSInfoTD()
        return [tdList[i+1].firstChild.data for i in range(len(tdList)) if i%7 == 2 and
                tdList[i].firstChild.data == domain]

    def setAllIP (self, ip):
        """Set same master IP address for all domains registered at puck dns service with this user account"""
        for domain in self.getDomains():
            self.setIP(domain, ip)
    
    def addDomain (self, ip, domain):
        """Add domain to puck DNS"""
        payload = {"domainname": domain, "masterip": ip, "submit": "Submit"}
        self.__makeRequest("/add", "Inserted new domain.", payload)

    def addDomains (self, ip, domainsList):
        """Add multiple domain entries at once"""
        data = {"domains": "\n".join(domainsList), "masterip": ip, "submit": "Submit"}
        self.__makeRequest("/bulk_add", "Domains inserted.", data)

    def delDomain(self, domain):
        """Delete domain from puck DNS"""
        self.__makeRequest(f"/delete/{domain}", f"{domain} successfully removed.")

    def delDomains(self, domainList):
        """Delete Multiple Domains"""
        for domain in domainList:
            self.delDomain(domain)

from . import _version
__version__ = _version.get_versions()['version']
